<?php

declare(strict_types=1);

namespace Drupal\simple_push_onesignal\Plugin\SimplePushBackend;

use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\onesignal\OneSignalService;
use Psr\Log\LoggerInterface;
use Drupal\simple_push\Attribute\SimplePushBackend;
use Drupal\simple_push\SimplePushBackendPluginBase;

/**
 * Plugin implementation of the simple_push_backend.
 */
#[SimplePushBackend(
  id: 'onesignal',
  label: new TranslatableMarkup('Onesignal'),
  description: new TranslatableMarkup('One Signal backend.'),
  settings: 'simple_push.backend.onesignal.settings'
)]
final class Onesignal extends SimplePushBackendPluginBase {
  /**
   * The OneSignal Service.
   *
   * @var \Drupal\onesignal\OneSignalService
   */
  protected $oneSignalService;

  /**
   * {@inheritdoc}
   * Construct a OneSignal  simple_push plugin object.
   *
   * @param \Drupal\onesignal\OneSignalService $oneSignalService
   *   The OneSignal Service.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The responsible for providing language support on language-unaware sites.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, RendererInterface $renderer, EntityFieldManagerInterface $entity_field_manager, ConfigFactoryInterface $config_factory, LoggerInterface $logger, LanguageManagerInterface $language_manager, OneSignalService $oneSignalService) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $renderer, $entity_field_manager, $config_factory, $logger, $language_manager);
    $this->setConfiguration($configuration);
    $this->oneSignalService = $oneSignalService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('renderer'),
      $container->get('entity_field.manager'),
      $container->get('config.factory'),
      $container->get('logger.channel.simple_push'),
      $container->get('language_manager'),
      $container->get('onesignal')
    );
  }

  /**
   *
   */
  public function addNotification(array $data):array {
    // Prepare the message
    // https://documentation.onesignal.com/reference/push-notification
    // https://documentation.onesignal.com/reference/rest-api-overview#create-notification
    // https://documentation.onesignal.com/reference/create-message
    // contents
    // https://www.computerminds.co.uk/articles/rendering-drupal-fields-right-way
    // $render_controller = $this->entityTypeManager->getViewBuilder($entity->getEntityTypeId());
    if (isset($data['entity'])) {
      $entity = $data['entity'];
      $site_config = $this->configFactory->get('system.site');
      $metatag = metatag_generate_entity_all_tags($entity);
      // $entity->title->view('teaser')
      $title = $metatag['title']['#attributes']['content'] ?? $entity->title->value;
      $contents = $metatag['description']['#attributes']['content'] ?? (string) $this->t("Read more on %name %slogan", [
        "%name" => $site_config->get('name'),
        "%slogan" => $site_config->get('slogan'),
      ]);
      $base_url = Url::fromUserInput('/', ['absolute' => TRUE])->toString();
      $picture = $metatag['og_image_0']['#attributes']['content'] ?? $base_url . theme_get_setting('logo.url');
      $url = $entity->toUrl()->setAbsolute()->toString();
      $msg_data = [
        "headings" => [
          "en" => $title,
        ],
        "contents" => [
        // $node->body->view('teaser'),
          "en" => $contents,
        ],
        "included_segments" => ["Total Subscriptions"],
        // \Drupal::theme()->getActiveTheme()->getLogo().
        "big_picture" => $picture,
        "chrome_web_image" => $picture,
        "url" => $url,
      ];

      $send_response = $this->oneSignalService->addNotification($msg_data);
      if ($send_response['_status_code'] / 100 != 2) {
        $this->logger->error('Error when trying to send message: @error_message', ['@error_message' => json_encode($send_response['errors'])]);
      }
    }
    return [];
  }

}
